<?php
namespace brewpress\app\CustomPostType;

class CreateCustomPostType {
	public $type;
	public $options = [];
	public $labels = [];
	public $messages = [];

	/**
	 * Creates a new CreateCustomPostType object
	 *
	 * @param string $type
	 * @param array $options
	 * @param array $labels
	 * @param array $messages
	 */
	public function __construct(
		$type,
		$labels = [],
		$options = [],
		$messages = []
	) {
		$this->type = $type;

		$brewpress   = new \brewpress\app\BrewPress();
		$text_domain = $brewpress->get_brewpress();

		$default_options = [
			'public'             => true,
			'publicly_queryable' => true,
			'query_var'          => true,
			'menu_icon'          => 'dashicons-star-filled',
			'rewrite'            => true,
			'capability_type'    => 'post',
			'hierarchical'       => false,
			'menu_position'      => null,
			'has_archive'        => true,
			'show_in_nav_menus'  => false,
			'show_ui'            => true,
			'show_in_menu'       => true,
			'supports'           => [ 'title', 'editor', 'thumbnail' ],
			'text_domain'        => $text_domain
		];
		$required_labels = [
			'singular_name' => ucwords( $this->type ),
			'plural_name'   => ucwords( $this->type )
		];

		$this->options           = $options + $default_options;
		$this->labels            = $labels + $required_labels;
		$this->options['labels'] = $labels + $this->default_labels();
		$this->messages          = $messages;
		add_action( 'init', array( $this, 'register_custom_post_type' ) );
		add_action( 'init', array( $this, 'filter_messages' ) );
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function getBrewpress() {
		return $this->brewpress;
	}

	/**
	 * Registers the content type using WP core function(s)
	 * @return null
	 */
	public function register_custom_post_type() {
		register_post_type( $this->type, $this->options );
	}

	/**
	 * Creates intelligent default labels from the required singular and plural labels
	 * @return array
	 */
	public function default_labels() {
		$text_domain = $this->options['text_domain'];

		return [
			'name'               => $this->labels['plural_name'],
			'singular_name'      => $this->labels['singular_name'],
			'add_new'            => __( 'Add New ' . $this->labels['singular_name'],
				'brewpress' ),
			'add_new_item'       => __( 'Add New ' . $this->labels['singular_name'],
				'brewpress' ),
			'edit'               => __( 'Edit', 'brewpress' ),
			'edit_item'          => __( 'Edit ' . $this->labels['singular_name'],
				'brewpress' ),
			'new_item'           => __( 'New ' . $this->labels['singular_name'],
				'brewpress' ),
			'all_items'          => __( 'All ',
					'brewpress' ) . $this->labels['plural_name'],
			'view'               => __( 'View ' . $this->labels['singular_name'] . ' Page',
				'brewpress' ),
			'view_item'          => __( 'View ' . $this->labels['singular_name'],
				'brewpress' ),
			'search_items'       => __( 'Search ' . $this->labels['plural_name'],
				'brewpress' ),
			'not_found'          => __( 'No matching ' . strtolower( $this->labels['plural_name'] ) . ' found',
				'brewpress' ),
			'not_found_in_trash' => __( 'No ' . strtolower( $this->labels['plural_name'] ) . ' found in Trash',
				'brewpress' ),
			'parent_item_colon'  => __( 'Parent ' . $this->labels['singular_name'],
				'brewpress' ),
			'menu_name'          => $this->labels['plural_name'],
		];
	}

	public function register_messages( $messages ) {
		$our_messages = $this->messages + $this->default_messages();

		$messages[ $this->type ] = $our_messages;

		return $messages;
	}

	public function filter_messages() {
		add_filter( 'post_updated_messages',
			array( $this, 'register_messages' ) );
	}


	public function default_messages() {
		global $post;

		$permalink   = get_permalink( $post );

		return [
			0  => '', // Unused. Messages start at index 1.
			1  => sprintf( __( $this->labels['singular_name'] . ' updated. <a href="%s">View ' . $this->labels['singular_name'] . '</a>',
				'brewpress' ),
				esc_url( $permalink ) ),
			2  => __( 'Custom field updated.', 'brewpress' ),
			3  => __( 'Custom field deleted.', 'brewpress' ),
			4  => __( $this->labels['singular_name'] . ' updated.',
				'brewpress' ),
			/* translators: %s: date and time of the revision */
			5  => isset( $_GET['revision'] ) ? sprintf( __( $this->labels['singular_name'] . ' restored to revision from %s',
				'brewpress' ),
				wp_post_revision_title( (int) $_GET['revision'],
					false ) ) : false,
			6  => sprintf( __( $this->labels['singular_name'] . ' published. <a href="%s">View ' . $this->labels['singular_name'] . '</a>',
				'brewpress' ),
				esc_url( $permalink ) ),
			7  => __( $this->labels['singular_name'] . ' saved.',
				'brewpress' ),
			8  => sprintf( __( $this->labels['singular_name'] . ' submitted. <a target="_blank" href="%s">Preview ' . $this->labels['singular_name'] . '</a>',
				'brewpress' ),
				esc_url( add_query_arg( 'preview', 'true', $permalink ) ) ),
			9  => sprintf( __( $this->labels['singular_name'] . ' scheduled for: <strong>%1$s</strong>. <a target="_blank" href="%2$s">Preview ' . $this->labels['singular_name'] . '</a>',
				'brewpress' ),
				// translators: Publish box date format, see http://php.net/date
				date_i18n( __( 'M j, Y @ H:i' ),
					strtotime( $post->post_date ) ),
				esc_url( $permalink ) ),
			10 => sprintf( __( $this->labels['singular_name'] . ' draft updated. <a target="_blank" href="%s">Preview ' . $this->labels['singular_name'] . '</a>',
				'brewpress' ),
				esc_url( add_query_arg( 'preview', 'true', $permalink ) ) ),
		];
	}
}