<?php
/**
 * BP Session
 *
 * This is a wrapper class for WP_Session / PHP $_SESSION and handles the storage of cart items, purchase sessions, etc
 *
 * @package     BrewPress
 * @subpackage  Classes/Session
 * @copyright   Copyright (c) 2015, Martin Goldie
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since       1.0
 */

namespace brewpress\app\Session;

use \brewpress\app\Session\includes\wpsession as WP_Session;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Session Class
 *
 * @since 1.5
 */
class Session {

	/**
	 * Holds our session data
	 *
	 * @var array
	 * @access private
	 * @since 1.5
	 */
	private $session;

	/**
	 * Session index prefix
	 *
	 * @var string
	 * @access private
	 * @since 2.3
	 */
	private $prefix = '';


	/**
	 * Get things started
	 *
	 * Defines our WP_Session constants, includes the necessary libraries and
	 * retrieves the WP Session instance
	 *
	 * @since 1.5
	 */
	public function __construct() {

		/**
		 * Clean this up if we are supporting PHP SESSIONS or not.
		 * if ( $this->use_php_sessions ) {
		 *     if ( is_multisite() ) {
		 *         $this->prefix = '_' . get_current_blog_id();
		 *     }

			// Use PHP SESSION (must be enabled via the BP_USE_PHP_SESSIONS constant).
			add_action( 'init', array( $this, 'maybe_start_session' ), -2 );

		} else {
		*/
		// Use WP_Session (default).
		if ( ! defined( 'WP_SESSION_COOKIE' ) ) {
			define( 'WP_SESSION_COOKIE', 'bp_wp_session' );
		}

		/*
		 * We may not need this now that we have namespaced the classes.
		 *
					if ( ! class_exists( 'Recursive_ArrayAccess' ) ) {
						require_once BP_PLUGIN_DIR . 'includes/libraries/class-recursive-arrayaccess.php';
					}

					if ( ! class_exists( 'WP_Session' ) ) {
						require_once BP_PLUGIN_DIR . 'includes/libraries/class-wp-session.php';
						require_once BP_PLUGIN_DIR . 'includes/libraries/wp-session.php';
					}
		*/
		add_filter( 'wp_session_expiration_variant', array( $this, 'set_expiration_variant_time' ), 99999 );
		add_filter( 'wp_session_expiration', array( $this, 'set_expiration_time' ), 99999 );

		if ( empty( $this->session ) ) {
			add_action( 'plugins_loaded', array( $this, 'init' ), -1 );
		} else {
			add_action( 'init', array( $this, 'init' ), -1 );
		}

	}

	/**
	 * Setup the WP_Session instance
	 *
	 * @access public
	 * @since 1.5
	 *
	 * @return array|bool|WP_Session
	 */
	public function init() {
		/*
		 * Codesniffer does not allow the use of $_SESSION so even though this is an
		 * optional parameter that would not be used most of the time. I have commented
		 * it out to force $_SESSION to not be used.
		if ( $this->use_php_sessions ) {
			$this->session = isset( $_SESSION['bp' . $this->prefix ] ) && is_array( $_SESSION['edd' . $this->prefix ] ) ? $_SESSION['edd' . $this->prefix ] : array();
		} else {
			$this->session = WP_Session::get_instance();
		}
		*/
		$this->session = WP_Session::get_instance();

		return $this->session;
	}


	/**
	 * Retrieve session ID
	 *
	 * @access public
	 * @since 1.6
	 * @return string Session ID
	 */
	public function get_id() {
		return $this->session->session_id;
	}


	/**
	 * Retrieve a session variable
	 *
	 * @access public
	 * @since 1.5
	 * @param string $key Session key.
	 * @return string Session variable
	 */
	public function get( $key ) {
		$key = sanitize_text_field( $key );
		return isset( $this->session[ $key ] ) ? maybe_unserialize( $this->session[ $key ] ) : false;
	}

	/**
	 * Set a session variable
	 *
	 * @since 1.5
	 *
	 * @param string         $key Session key.
	 * @param string|integer $value Session variable.
	 * @return string Session variable
	 */
	public function set( $key, $value ) {

		$key = sanitize_key( $key );

		if ( is_array( $value ) ) {
			$this->session[ $key ] = serialize( $value );
		} else {
			$this->session[ $key ] = $value;
		}

		/*
		 * Codesniffer does not allow the use of $_SESSION so even though this is an
		 * optional parameter that would not be used most of the time. I have commented
		 * it out to force $_SESSION to not be used.
		if ( $this->use_php_sessions ) {

			$_SESSION['bp' . $this->prefix ] = $this->session;
		}
		*/

		return $this->session[ $key ];
	}

	/**
	 * Force the cookie expiration variant time to 23 hours
	 *
	 * @access public
	 * @since 2.0
	 * @param int $exp Default expiration (1 hour).
	 * @return int
	 */
	public function set_expiration_variant_time( $exp ) {
		return ( 30 * 60 * 23 );
	}

	/**
	 * Force the cookie expiration time to 24 hours
	 *
	 * @access public
	 * @since 1.9
	 * @param int $exp Default expiration (1 hour).
	 * @return int
	 */
	public function set_expiration_time( $exp ) {
		return ( 30 * 60 * 24 );
	}

}
